(*
This file is part of Ceve.

Ceve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ceve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ceve; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

(**
Module to output data in CNF format (for either tart or a SAT solver
@author Jaap Boender
*)

open Metadata

let output_tart_dependencies (ht: Dependencies.pkg_hashtable) (pkg: pkg_metadata): unit =
let arch_of (a: pkg_architecture): string =
begin
	match a with
		Specific s -> s
	| AllArchitectures -> "all"
	| AnyArchitecture -> "any"
	| Source -> "source"
end in
let var_name (n: string) (v: string) (a: pkg_architecture): string =
begin
	n ^ "'" ^ v ^ "@" ^ (arch_of a)
end in
begin
	let dep_str = Tools.uniq (List.sort compare
	(List.map (fun disj ->
	let deps = List.flatten (List.map (fun dep -> 
		List.map
			(fun (v, m) -> var_name dep.target v m.architecture)
		(List.filter
			(fun (v, m) -> Dependencies.can_provide (Equal v) dep.version_spec)
		(Hashtbl.find_all ht dep.target)))
	disj) in
		if deps <> [] then
			"!" ^ (var_name pkg.name pkg.version pkg.architecture) ^ " " ^ (String.concat " " (Tools.uniq (List.sort compare deps))) ^ "\n"
		else
			""
	) (pkg.depends @ pkg.pre_depends))) in
	let cfl_str = Tools.uniq (List.sort compare (List.flatten (List.map (fun cfl ->
		(List.map
			(fun (v, m) -> "!" ^ (var_name pkg.name pkg.version pkg.architecture) ^ " !" ^ (var_name cfl.target v m.architecture))
		(List.filter
			(fun (v, m) -> Dependencies.can_provide (Equal v) cfl.version_spec && m.name <> pkg.name)
		(Hashtbl.find_all ht cfl.target)))
	) pkg.conflicts))) in
		if ((dep_str = []) || (dep_str = [""])) && (cfl_str = []) then
			output_string !Options.output_channel ((var_name pkg.name pkg.version pkg.architecture) ^ " !" ^ (var_name pkg.name pkg.version pkg.architecture) ^ "\n")
		else
		begin
			if dep_str <> [] then
				output_string !Options.output_channel (String.concat "" dep_str);
			if cfl_str <> [] then
				output_string !Options.output_channel ((String.concat "\n" cfl_str) ^ "\n")
		end
end;;

let output_tart_cnf (pkgs: pkg_metadata list): unit =
let ht = Dependencies.create_pkgs_hashtable pkgs in
begin
	prerr_endline "[CNF] Outputting Tart CNF lines...";
	List.iter (output_tart_dependencies ht) pkgs
end;;
