(*
This file is part of Ceve.

Ceve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ceve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ceve; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

(**
General metadata module
@author Jaap Boender
*)

(** Type for architecture specifications *)
type pkg_architecture =
| Specific of string
| AllArchitectures (** Architecture-independent package *)
| AnyArchitecture (** Package for building on any architecture *)
| Source (** Source package *)

type pkg_version_spec =
| AnyVersion
| Less of string
| LessEqual of string
| Equal of string
| GreaterEqual of string
| Greater of string

type pkg_dependency = 
{
	target: string;
	version_spec: pkg_version_spec
}

(** Type for dependency disjunctions *)
type pkg_disjunction = pkg_dependency list

(** Type for dependency expressions (i.e. a list of disjunctions) *)
type pkg_dep_expression = pkg_disjunction list

(** Type for (Debian) priority levels *)
type pkg_priority = Required | Important | Standard | Optional | Extra

(** Type for package metadata *)
type pkg_metadata =
{
	architecture: pkg_architecture;
	bugs: string;
	build_essential: bool;
	conflicts: pkg_dependency list;
	depends: pkg_dep_expression;
	description: string;
	distribution: string;
	enhances: pkg_dep_expression;
	essential: bool;
	filename: string;
	installed_size: int32;
	license: string;
	maintainer: string;
	md5sum: string;
	name: string;
	origin: string;
	os: string;
	pre_depends: pkg_dep_expression;
	priority: pkg_priority;
	provides: pkg_dependency list;
	python_version: string;
	recommends: pkg_dep_expression;
	replaces: pkg_dependency list;
	section: string;
	sha1sum: string;
	sha256sum: string;
	size: int32;
	source_package: string;
	source_version: string;
	suggests: pkg_dep_expression;
	summary: string;
	tags: string list;
	tasks: string list;
	url: string;
	vendor: string;
	version: string;
	unknown: (string * string) list;
	files: (string * string) list
}

let empty_metadata = 
{architecture = AllArchitectures; bugs = ""; build_essential = false;
	conflicts = []; depends = []; description = ""; distribution = "";
	enhances = []; essential = false; filename = ""; installed_size = Int32.zero;
	license = ""; maintainer = ""; md5sum = ""; name = ""; origin = ""; os = "";
	pre_depends = []; priority = Standard; provides = []; python_version = "";
	recommends = []; replaces = []; section = ""; sha1sum = ""; sha256sum = "";
	size = Int32.zero; source_package = ""; source_version = ""; suggests = [];
	summary = ""; tags = []; tasks = []; url = ""; vendor = ""; version = "";
	unknown = []; files = []}
