(*
This file is part of Ceve.

Ceve is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Ceve is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Ceve; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

(**
Pretty print module
@author Jaap Boender
*)

open Metadata

let format_dependency (dep: pkg_dependency): string =
begin
	dep.target ^ match dep.version_spec with
	| AnyVersion -> ""
	| Less v -> " (<< " ^ v ^ ")"
	| LessEqual v -> " (<= " ^ v ^ ")"
	| Equal v -> " (= " ^ v ^ ")"
	| GreaterEqual v -> " (>= " ^ v ^ ")"
	| Greater v -> " (>> " ^ v ^ ")"
end;;

let out_string = output_string !Options.output_channel

let pretty_print_dep_expression (dep_expr: pkg_dep_expression): unit =
begin
  out_string (String.concat ", " (List.map (fun disj -> String.concat " | " (List.map format_dependency disj)) dep_expr) ^ "\n")
end;;

let pretty_print_package (pkg: pkg_metadata): unit =
begin
	out_string ("Package: " ^ pkg.name ^ " (" ^ pkg.version ^ ")\n");
	out_string ("Architecture: " ^ (match pkg.architecture with
	| AnyArchitecture -> "any" 
	| AllArchitectures -> "all"
	| Source -> "source package"
	| Specific s -> s) ^ "\n");
	out_string ("Depends: "); pretty_print_dep_expression pkg.depends;
	out_string ("Recommends: "); pretty_print_dep_expression pkg.recommends;
	out_string ("Pre-Depends: "); pretty_print_dep_expression pkg.pre_depends;
	out_string ("Suggests: "); pretty_print_dep_expression pkg.suggests;
	out_string ("Conflicts: " ^ (String.concat ", " (List.map format_dependency pkg.conflicts)) ^ "\n");
	out_string ("Replaces: " ^ (String.concat ", " (List.map format_dependency pkg.replaces)) ^ "\n");
	out_string ("Provides: " ^ (String.concat ", " (List.map format_dependency pkg.provides)) ^ "\n");
	out_string ("Size: " ^ (Int32.to_string pkg.size) ^ "\n");
	out_string ("Installed-Size: " ^ (Int32.to_string pkg.installed_size) ^ "\n");
	out_string ("Files: " ^ (String.concat ", " (List.map (fun (f, s) -> f ^ " (" ^ s ^ ")") pkg.files)) ^ "\n");
	flush !Options.output_channel
end;;

let pretty_print (pkgs: pkg_metadata list): unit =
begin
	prerr_endline ("[PP] Pretty printing package metadata (" ^ (string_of_int (List.length pkgs)) ^ " packages)...");
	List.iter pretty_print_package pkgs
end;;
